
 
#include "common.hxx" 			// a common header file
#include "wall.hxx"			// the wall header
#include "orb.hxx"			// the orb header
#include "ped.hxx"			// the pedestal header
#include "tele.hxx"			// the teleporter header
#include "space.hxx"			// the space header
#include "viewer.hxx"			// the viewer header
#include "door.hxx"			// the door header
#include "start.hxx"			// the start header
#include "otto.hxx"			// the otto header
#include "textdisplay.hxx"		// the display header
#include "zorro.hxx"			// the zorro header
#include "flutter.hxx"			// the flutter header


// these are variables to use for the different textures
GLuint  TEXTURE_1;
GLuint  TEXTURE_2;
GLuint  TEXTURE_3;

/* instantiate objects here */
/****************************/

char theMaze[MAX_LENGTH][MAX_LENGTH];			// the maze
struct							// a structure of flags	
{
	int firstTime;					// indicates first run of the program
	int moveForward;				// indicates that 'i' key was pressed
	int turnLeft;					// indicates that 'j' key was pressed
	int turnRight;					// indicates that 'l' key was pressed
	int seeFromTop;					// indicates if the user wants to switch to the map mode
	int enableFog;					// indicates if the fog is turned on	
	int enableTexture;				// indicates that the textures on the walls and floor should be on	
	int tp; 					// indicates if the 't' key was pressed
	int mp;						// indicates that the 'm' key was pressed	
	int fp;						// indicates if the 'f' key was pressed
}flags;

GLfloat light_position[4];

//vectors of different classes			
vector<wall> wallVector;			
vector<door> doorVector;
vector<orb> orbVector;
vector<ped> pedVector;
vector<teleporter> teleporterVector;
vector<space> spaceVector;
vector<start> startVector;
vector<zorro> zorroVector;


// the camera
viewer camera;			

// otto
otto ottoObj[3];

// flutter
flutter flutterObj[3];

// display
textdisplay orbDisplay;
textdisplay ottoDisplay;

/*************************************************************/
/*							     */  
/* Read in the maze from the file maze.txt.		     */	
/* Store it in a 2D array. 				     */	
/*							     */  
/*************************************************************/
void readMaze()
{

	for ( int k =0; k < MAX_LENGTH; k++)
		for (int l =0 ; l< MAX_LENGTH ; l++)
			theMaze[k][l] = ' ';	
	
	FILE* fp = fopen(mazefile,"r");			// open the maze file in read mode
	if (!fp) {					// exit if error in opening file
		printf("Couldn't open mazefile file \n");		
		exit(1);
	}
	
	int i=0;					// variables to access the maze array.
	int j=0;					// i is the cartesian x direction
 	while(!feof(fp))				// j is the cartesian y direction
	{
		char tmp;
		fscanf(fp,"%c",&tmp);			// read in the file
		if (tmp == '\n')			
		{
			i++;
			j=0;
		}
		else
		{
			theMaze[i][j] = tmp;
			j++;		    	
		}
	}

}

/*************************************************************/
/*							     */  
/* This routine reads a texture file which is stored	     */	
/* as a .RAW file. It also sets up some other functions	     */	
/* used by OpenGL to do texture mapping			     */  
/*************************************************************/
int loadTextureFile (char *fileName, int imageWidth, int imageHeight, int id)
{
    unsigned char *imageData = NULL;
    int size;
    FILE *fp;

    if (!(fp = fopen (fileName, "rb")))
        return 0;
    size = imageWidth * imageHeight; 

    imageData = (unsigned char*)malloc (size * 3); 

    if (imageData == NULL)
        return 0;

    int numBytes = fread (imageData, sizeof (unsigned char), size * 3, fp);

    if (numBytes != size * 3)
    {
        free (imageData);
        return 0;
    }

    fclose (fp);

    glBindTexture (GL_TEXTURE_2D, id);
    glPixelStorei (GL_UNPACK_ALIGNMENT, 1);
    glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    //glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST); 
    glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    //glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST); 
    glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
    glTexImage2D (GL_TEXTURE_2D, 0, GL_RGB, imageWidth, imageHeight, 0, GL_RGB, GL_UNSIGNED_BYTE, imageData);

    /* release data, its been uploaded */
    free (imageData);

    return 1;
}

/*************************************************************/
/*							     */  
/* This function returns the distance of an object	     */
/* from the camera given it's location in the maze.	     */  
/*************************************************************/
float calculateDistance(int currentX, int currentY)
{
	float viewerX = camera.getEyesX();		//get the camera's location
	float viewerY = camera.getEyesY();

	float diffX = viewerX - ((currentX * 5.0) + 2.5);	// calculate the difference in coordinates
	float diffY = viewerY - ((currentY * 5.0) + 2.5);

	float distance = sqrt((diffX * diffX) + (diffY * diffY));

	return distance;
}

/*************************************************************/
/*							     */  
/* Draw the floor. 					     */
/* 							     */	
/*************************************************************/
void drawFloor(int x, int y, int status)
{
	if (flags.firstTime)
	{
		space spaceObj;			// create floor (space) object
		spaceObj.drawSpace(x, y, status);	// draw it
		spaceVector.push_back(spaceObj);// store it
	}
	else
	{	// for all the spaces
		for (vector<space>::iterator spaceCounter = spaceVector.begin(); spaceCounter != spaceVector.end(); spaceCounter++)
			if (((*spaceCounter).getPositionX() == x) && ((*spaceCounter).getPositionY() == y))
				(*spaceCounter).drawSpace(x,y, status);			// draw floor
	}
}

/*************************************************************/
/*							     */  
/* Draw the door. 					     */
/* If the door is on the boundary, check that the viewer does*/
/* not slip out of the maze. Do this by drawing a transparent*/  	
/* wall where the door is located.			     */
/*************************************************************/
void drawDoor(int x, int y)
{
	if (flags.firstTime)
	{
		door doorObj;			// door object
		if ((theMaze[x-1][y+1] == ' ') || (theMaze[x+1][y+1] == ' ') || (theMaze[x+1][y-1] == ' ') || (theMaze[x-1][y-1] == ' '))		
			doorObj.setAngle();	// if the door is on the periphery rotate it by 90 degrees.

		if (theMaze[x][y-1] == '#' && theMaze[x][y+1] == '#')
			doorObj.setAngle();	// if the door is on the periphery rotate it by 90 degrees.			
		doorObj.drawDoor(x, y);		// draw the door
		doorVector.push_back(doorObj);	// store it
		
		// Check if the door is on the boundary, ie. check if there are spaces around the door.
		// if yes then create a transparent wall on the position.
		if ((theMaze[x-1][y+1] == ' ') || (theMaze[x+1][y+1] == ' ') || (theMaze[x+1][y-1] == ' ') || (theMaze[x-1][y-1] == ' '))
		{
			wall wallObj;		// create a wall
//			wallObj.setWallIsDoor();// set a flag to indicate that the wall is actually a door that does not open!
			wallObj.drawWall(x,y, FALSE);	// draw the wall
			wallObj.darkestColor();	// darken it 
			wallVector.push_back(wallObj);// store it
		}	

	}
	else
	{	// for all the doors
		for (vector<door>::iterator doorCounter = doorVector.begin(); doorCounter != doorVector.end(); doorCounter++)
			if (((*doorCounter).getPositionX() == x) && ((*doorCounter).getPositionY() == y))
				(*doorCounter).drawDoor(x,y);			// draw the door
	}
}

/*************************************************************/
/*							     */  
/* Draw the orb. 					     */
/*							     */  	
/*************************************************************/
void drawOrb(int x, int y)
{
	if (flags.firstTime)
	{
		orb orbObj;			// an orb object
		orbObj.drawOrb(x, y);		//draw it
		orbVector.push_back(orbObj);	//store it
	}
	else
	{
		for (vector<orb>::iterator orbCounter = orbVector.begin(); orbCounter != orbVector.end(); orbCounter++)
			if (((*orbCounter).getPositionX() == x) && ((*orbCounter).getPositionY() == y))
				if (!(*orbCounter).getOrbTaken())
					(*orbCounter).drawOrb(x,y);			// draw the orb
	}

}

/*************************************************************/
/*							     */  
/* Draw the pedestal. 					     */
/*							     */  	
/*************************************************************/
void drawPedestal(char object, int x, int y)
{
	if (flags.firstTime)
	{
		ped pedObj;			// a pedestal object
		pedObj.drawPed(x, y);		// draw the pedestal
		if (object == 'P')		// if only a pedestal is to be drawn then 
		{
			pedObj.setOrbTaken();   // make it red as if an orb was taken away
			pedObj.makeRed();
		}
		pedVector.push_back(pedObj);	// store it
	}
	else
	{	// check all the pedestals
		for (vector<ped>::iterator pedCounter = pedVector.begin(); pedCounter != pedVector.end(); pedCounter++)
			if (((*pedCounter).getPositionX() == x) && ((*pedCounter).getPositionY() == y))
				(*pedCounter).drawPed(x,y);				// draw the pedestal
	}
}

/*************************************************************/
/*							     */  
/* Draw the teleporter. 				     */
/*							     */  	
/*************************************************************/
void drawTeleporter(int x, int y, char teleNo)
{
	if (flags.firstTime)
	{
		teleporter teleporterObj;			// a teleporter object
		teleporterObj.setTeleNo(teleNo);
		teleporterObj.drawTeleporter(x, y);		// draw Teleporter 
		teleporterVector.push_back(teleporterObj);	// store it
	}
	else
	{	// for all the teleporters
		for (vector<teleporter>::iterator teleporterCounter = teleporterVector.begin(); teleporterCounter != teleporterVector.end(); teleporterCounter++)
			if (((*teleporterCounter).getPositionX() == x) && ((*teleporterCounter).getPositionY() == y))
				(*teleporterCounter).drawTeleporter(x,y);	  // draw the teleporter
	}
}

/*************************************************************/
/*							     */  
/* Draw the Starting Position. Initialize the camera position*/
/*							     */  	
/*************************************************************/

void startHere(int x, int y)
{
	if (flags.firstTime)				
	{
		camera.initialize(x, y);	// place the camera at this position
		start startObj;			// draw the starting position and 
		startObj.drawStart(x, y);	// store it in a vector	
		startVector.push_back(startObj);
	}
	else
	{	// for all objects of the start class
		for (vector<start>::iterator startCounter = startVector.begin(); startCounter != startVector.end(); startCounter++)
			if (((*startCounter).getPositionX() == x) && ((*startCounter).getPositionY() == y))
				(*startCounter).drawStart(x,y);			// draw the start position
	}
}


/*************************************************************/
/*							     */  
/* Draw the Wall. The walls within 10 feet of the camera     */	
/* will appear brighter than those beyond 10 feet. If the    */	
/* wall is 110 feet away , it will be completely dark.	     */ 
/*							     */  	
/*************************************************************/
void drawWall(int x, int y, int status)
{
	if (flags.firstTime)
	{
		wall wallObj;
		wallObj.drawWall(x, y, status);
		wallVector.push_back(wallObj);
	}
	else
	{
		for (vector<wall>::iterator wallCounter = wallVector.begin(); wallCounter != wallVector.end(); wallCounter++)
			if (((*wallCounter).getPositionX() == x) && ((*wallCounter).getPositionY() == y))
				(*wallCounter).drawWall(x,y, status);
	}
}

void drawZorro(int x, int y)
{
	if (flags.firstTime)
	{
		zorro zorroObj;
		zorroObj.init(x,y);
		zorroObj.drawZorro();
		zorroVector.push_back(zorroObj);
	}
	else
	{
		for (vector<zorro>::iterator zorroCounter = zorroVector.begin(); zorroCounter != zorroVector.end(); zorroCounter++)
			if (((*zorroCounter).getPositionX() == x) && ((*zorroCounter).getPositionY() == y))
				if ((*zorroCounter).getInactive() == FALSE)
					(*zorroCounter).drawZorro();
	}
	
}

/*************************************************************/
/*							     */  
/* Draw the Wall. The walls within 10 feet of the camera     */	
/* will appear brighter than those beyond 10 feet. If the    */	
/* wall is 110 feet away , it will be completely dark.	     */ 
/*							     */  	
/*************************************************************/
void drawFlutter(int x, int y)
{
	
	if (flags.firstTime)
	{
		int fNo;
		if (x == 14 && y == 22) fNo = 0;
		if (x == 17 && y == 14) fNo = 1;
		if (x == 17 && y == 30) fNo = 2;
	
		flutterObj[fNo].drawFlutter(x, y, fNo);
		flutterObj[fNo].setNotFirstTime();
	}
	else
	{
		for (int i = 0; i< 3; i++)
			if ((flutterObj[i].getPositionX() == x) && (flutterObj[i].getPositionY() == y))
				flutterObj[i].drawFlutter(x,y, i);

	}/* end of if*/
}

	
/*************************************************************/
/*							     */  
/* Check if the viewer has walked over an orb.	             */	
/* The orb will disappear when collected.		     */	 
/*							     */  	
/*************************************************************/
void collideWithOrb(int currentX, int currentY)
{
	// check all the orbs
	for (vector<orb>::iterator orbCounter = orbVector.begin(); orbCounter != orbVector.end(); orbCounter++)
	{
		// if an orb position matches the current position of the viewer
		if (((*orbCounter).getPositionX() == currentX) && ((*orbCounter).getPositionY() == currentY))
		{
			if ((*orbCounter).getOrbTaken() == FALSE)
			{
				orbDisplay.increaseOrbCtr();
				#ifdef WIN32
					PlaySound("beep_arcade.au",NULL,SND_ASYNC);
				#else
					system("/usr/sbin/sfplay beep_arcade.au &");
				#endif
			}
			(*orbCounter).setOrbTaken();		// the orb is taken
			// locate the pedestal drawn on the same spot		

			for (vector<ped>::iterator pedCounter = pedVector.begin(); pedCounter != pedVector.end(); pedCounter++)
				if (((*pedCounter).getPositionX() == currentX) && ((*pedCounter).getPositionY() == currentY))
				{
					(*pedCounter).setOrbTaken();	// since orb is taken 
					(*pedCounter).makeRed();	// make the pedestal red in color.	
				}
		}
	}
	if ((currentX == 0) || (currentY == 0))
		camera.oldXoldY();
}


/*************************************************************/
/*							     */  
/* Check if the viewer has walked over a teleporter.         */	
/* If yes, you should get transported to a safe free position*/
/*in the maze.						     */	 
/*							     */  	
/*************************************************************/
void collideWithTeleporter(int currentX, int currentY, char teleNo)
{
	int done = FALSE;
	// check all the teleporters
	char nextTeleNo = teleNo + 1;
	if (teleNo == '9')
		nextTeleNo = '0';

	for (vector<teleporter>::iterator teleCounter = teleporterVector.begin(); teleCounter != teleporterVector.end() && (!done); teleCounter++)
	{	
		if ((*teleCounter).getTeleNo() == nextTeleNo)
		{	
			#ifdef WIN32
				PlaySound("bubble.au", NULL, SND_ASYNC);
			#else
				system("/usr/sbin/sfplay Bubble.au &");
			#endif	
			camera.changeEyes((*teleCounter).getPositionX(), (*teleCounter).getPositionY());
			done = TRUE;
		}
	}
} 

/*************************************************************/
/*							     */  
/* Check if the viewer has collides with a wall              */	
/* If yes, you should slide along the wall,until a corner    */
/* is reached.						     */	 
/*							     */  	
/*************************************************************/
void collideWithAllWalls()
{
	float oldEyesX = camera.getCurrentEyesX();	// get the previous and next position 
	float oldEyesY = camera.getCurrentEyesY();      // of the camera.
	float newEyesX = camera.getNextEyesX();
	float newEyesY = camera.getNextEyesY();

	int oX = oldEyesX/5.0;				// calculate the previous and next position 
	int oY = oldEyesY/5.0;				// of the camera in the maze ie. get it's
	int nX = newEyesX/5.0;				// location in the 2D array.
	int nY = newEyesY/5.0;

	int oldXnewY = FALSE;				// set flags to false
	int newXoldY = FALSE;

	// check all the walls
	for (vector<wall>::iterator wallCounter = wallVector.begin(); wallCounter != wallVector.end(); wallCounter++)
	{
		// if the next position collides with any wall in the maze
		if ((newEyesX <= (*wallCounter).getRightEdge()) && 
		    (newEyesX >= (*wallCounter).getLeftEdge())  &&
		    (newEyesY >= (*wallCounter).getBottomEdge()) &&	 
		    (newEyesY <= (*wallCounter).getTopEdge()))
			{
				#ifdef WIN32
					PlaySound("ding.au",NULL,SND_ASYNC);
				#else
					system("/usr/sbin/sfplay ding.au &");
				#endif
				// check if there is a wall in the X direction	
				// if yes then set oldXnewY to true
				// so that you slide along the wall in the Y direction
				if (theMaze[oX][nY] != '#')
					oldXnewY = TRUE;

				// check if there is a wall in the Y direction	
				// if yes then set newXoldY to true
				// so that you slide along the wall in the X direction
				if (theMaze[nX][oY] != '#')
					newXoldY = TRUE;

				// if you have reached a corner stay put
				// set both flags
				if ((theMaze[oX][nY] == '#') && (theMaze[nX][oY] == '#'))
				{
					oldXnewY = TRUE;
					newXoldY = TRUE;
				}
			}
	}

	if ((oldXnewY) && (newXoldY)) 	// if both flags are set
		camera.oldXoldY();	// go back to previous position	before collision		
	else
	{
		if (oldXnewY) camera.oldXnewY();	// move in the direction set 
		if (newXoldY) camera.newXoldY();
	}
}

/*************************************************************/
/* This routine positions the Otto in a suitable place in    */	
/* the maze. The position is calculated randomly.	     */  
/* 							     */	
/*************************************************************/
void positionOtto(int i)
{
		int xPos=0, yPos=0;
		while (theMaze[xPos][yPos]!='.')
		{ 
			xPos =  99*rand()/RAND_MAX;
			yPos =  99*rand()/RAND_MAX;
		}
		theMaze[xPos][yPos] = '@';
		ottoObj[i].init(xPos,yPos);
		for ( i = 0;i<500;i++)
			for(int j=0;j<500;j++)
				if (theMaze[i][j] == '@')
					theMaze[i][j] = '.';
}


/*************************************************************/
/* This routine checks if any otto object collides with      */	
/* the camera.If so then increment the display		     */  
/* 							     */	
/*************************************************************/
void didOttoCollideWithCamera(int i)
{

	int currOttoX = (ottoObj[i].getCurrX()/5);
	int currOttoY = (ottoObj[i].getCurrY()/5);	
	int currCameraX = camera.getEyesX()/5;
	int currCameraY = camera.getEyesY()/5;

	if (currCameraX == currOttoX && currCameraY == currOttoY)
	{

		#ifdef WIN32
			PlaySound("beep_drop_of_water.au",NULL,SND_ASYNC);
		#else
			system("/usr/sbin/sfplay beep_drop_of_water.au &");
		#endif;
		if (!ottoObj[i].getZorroTaken())	
		{
			positionOtto(i);
			ottoDisplay.increaseOttoCtr();	   
		}
		else
			ottoObj[i].resetZorroTaken();	

	}			
}

/*************************************************************/
/* This routine checks if any otto object collides with      */	
/* a sending teleporter. If so it gets teleported	     */  
/* 							     */	
/*************************************************************/
void didOttoCollideWithTeleporter(int i)
{
	int nextX = (ottoObj[i].getNextX()/5);
	int nextY = (ottoObj[i].getNextY()/5);
	char nextObj = theMaze[nextX][nextY];
	

	if ( nextObj == '1' ||
	     nextObj == '3' ||	
	     nextObj == '5' ||	
	     nextObj == '7' ||	
	     nextObj == '9')
	{
		char nextTeleNo = nextObj + 1;
		for (int k = 0; k < MAX_LENGTH; k++)
			for (int j = 0; j < MAX_LENGTH; j++)
				if (theMaze[k][j] == nextTeleNo)
				{		
					ottoObj[i].init(k,j);
					break;
				}
	}

}


/*************************************************************/
/* This routine checks if any otto object collides with      */	
/* a wall. If so it bounces off the wall		     */  
/* 							     */	
/*************************************************************/
void didOttoCollideWithWall(int i)
{
	int currX = (ottoObj[i].getCurrX()/5);
	int currY = (ottoObj[i].getCurrY()/5);	
	int nextX = (ottoObj[i].getNextX()/5);
	int nextY = (ottoObj[i].getNextY()/5);

	if (nextX > 500 || nextX < 0 || nextY > 500 || nextY < 0)
		positionOtto(i);
	else
	{

	// if the next position in the maze is a wall then bounce off
	// set the appropriate direction for the otto object
	// check all the surrounding spaces
	if (theMaze[nextX][nextY] == '#')
	{
		if (currX == nextX)
		{
			if (nextY == currY + 1)	
				ottoObj[i].setGoDown();
			else
				if (nextY == currY - 1)
					ottoObj[i].setGoUp();
		}
		if (currY == nextY)
		{
			if (nextX == currX + 1)	
				ottoObj[i].setGoLeft();
			else
				if (nextX == currX - 1)
					ottoObj[i].setGoRight();
		}
		if (nextX == currX - 1 && nextY == currY -1)
		{
			ottoObj[i].setGoUp();
			ottoObj[i].setGoRight();	
		}
		if (nextX == currX + 1 && nextY == currY - 1)
		{
			ottoObj[i].setGoUp();
			ottoObj[i].setGoLeft();
		}
		if (nextX == currX + 1 && nextY == currY + 1)
		{
			ottoObj[i].setGoDown();
			ottoObj[i].setGoLeft();
		}
		if (nextX == currX - 1 && nextY == currY + 1)
		{
			ottoObj[i].setGoDown();
			ottoObj[i].setGoRight();
		}
	}
	}
}


/*************************************************************/
/* This routine checks if the player collides with	     */	
/* a zorro.If so then halt the motion of the ottos	     */  
/* 							     */	
/*************************************************************/
void collideWithZorro(int currentX, int currentY)
{
	for (vector<zorro>::iterator zorroCounter = zorroVector.begin(); zorroCounter != zorroVector.end(); zorroCounter++)
		// if the next position collides with any zorro in the maze
		// make that zorro inactive so that it is not drawn anymore
		if (currentX == (*zorroCounter).getPositionX() && currentY == (*zorroCounter).getPositionY())
		{
			if ((*zorroCounter).getInactive() == FALSE)
			{	
				(*zorroCounter).setInactive();
				#ifdef WIN32
					PlaySound("cymbal.au",NULL,SND_ASYNC);
				#else
					system ("/usr/sbin/sfplay cymbal.au &");
				#endif
				camera.setDrawZorro();
				for (int i = 0; i < 3; i++)
					ottoObj[i].setZorroTaken();
			}
		}

}


/*************************************************************/
/* This routine makes the correct flutter object 	     */	
/* travel a path to a teleporter which takes you to a        */
/* different world camera.If so then increment the display   */  
/* 							     */	
/*************************************************************/
void changeFlutter(int fNo)
{
		flutterObj[fNo].moveFlutter();
}

/*************************************************************/
/*							     */  
/* Enable the fog. All the fog parameters are described	     */	
/* here							     */	
/*************************************************************/
void enableFog()
{

	GLfloat fogColor[4]= {0.0f, 0.0f, 0.0f, 1.0f};
	glClearColor(0.0f,0.0f,0.0f,1.0f);                      // Clear To The Color Of The Fog 

	glFogi(GL_FOG_MODE, GL_LINEAR);			        // Fog Mode
	glFogfv(GL_FOG_COLOR, fogColor);                        // Set Fog Color
	glFogf(GL_FOG_DENSITY, 0.05f);                          // How Dense Will The Fog Be
	glHint(GL_FOG_HINT, GL_DONT_CARE);                      // Fog Hint Value
	glFogf(GL_FOG_START, 10.0f);                            // Fog Start Depth
	glFogf(GL_FOG_END, 110.0f);                             // Fog End Depth
	glEnable(GL_FOG);                                       // Enables GL_FOG

}

/*************************************************************/
/* keyboard						     */	
/*							     */  
/* deal with the keyboard input to the program.  	     */	
/* 							     */	
/*************************************************************/

void keyboard(unsigned char userkey, int x, int y)
{
	// set appropriate flag
	switch(userkey)
	{
		case 27:
			exit(-1);
			break;
		case 'i':
		case 'I':
			flags.moveForward = TRUE;	
			break;
		case 'j':
		case 'J': 
			flags.turnLeft = TRUE;
			break;
		case 'l':
		case 'L':
			flags.turnRight = TRUE;
			break;
		case 'm':
		case 'M':
			if (flags.mp)
			{
				flags.seeFromTop = FALSE;
				flags.mp = FALSE;
			}
			else	
			{
				flags.seeFromTop = TRUE;
				flags.mp = TRUE;
			}
			break;
		case 'f':
		case 'F':
			if (flags.fp)
			{
				flags.enableFog = FALSE;
				flags.fp = FALSE;
			}
			else	
			{
				flags.enableFog = TRUE;
				flags.fp = TRUE;
			}
			break;
		case 't':
		case 'T':
			if (flags.tp)
			{
				flags.enableTexture = FALSE;
				flags.tp = FALSE;
			}
			else	
			{
				flags.enableTexture = TRUE;
				flags.tp = TRUE;
			}
			break;
		default:
			break;	
	}
}

/*************************************************************
 *
 * update
 * 
 * update everything that needs to be updated in the scene - this
 * function should be called via a timer to ensure that updates 
 * happen at a regular rate, independent of the graphics update
 * rate
 *
 *************************************************************/

void update(int value)
    {
   
    /* at the end of this function
       reinstate the timer callback */
    /********************************/

	for (int i = 0; i < 3; i++)
	{
		if (!ottoObj[i].getZorroTaken())
		{
			ottoObj[i].moveOtto();
			didOttoCollideWithWall(i);
			didOttoCollideWithTeleporter(i);
		}
 		didOttoCollideWithCamera(i);	
	}

	// always keep the light at the camera position
	// so that even when you move to map mode you see
	// the light with the user representation	
	light_position[0] = camera.getEyesX();
	light_position[1] = camera.getEyesY();
	light_position[2] = camera.getEyesZ();
	light_position[3] = 1;


	// check if the player is within 10 feet of a flutter object
	for (i = 0; i < 3; i++)
		if (calculateDistance(flutterObj[i].getNextX(), flutterObj[i].getNextY()) <= 10)
			changeFlutter(i);

	// i key was pressed
	if (flags.moveForward)
	{
		camera.moveForward();				// calculate the new position of the camera		
		int currentX = camera.getCurrentPositionX();	// obtain the previous position
		int currentY = camera.getCurrentPositionY();	// and the next position of the camera in the array
		int nextX = camera.getNextPositionX();
		int nextY = camera.getNextPositionY();	
		collideWithAllWalls();				// check if there are any collisions with the walls
		

		char nextObject = theMaze[nextX][nextY];	// find out what the object in the next position is
		switch (nextObject)					
		{
			case 'O' : 				// if the next object is an Orb 
				collideWithOrb(currentX, currentY);
				break;
			case '1':				// if the next object is a Teleporter
			case '3':
			case '5':
			case '7':
			case '9':
				collideWithTeleporter(currentX, currentY, nextObject);
				break;
			case 'D':				// if there is a door situated on the periphery
				if ((currentX == 0) || (currentY == 0))
					camera.oldXoldY();	// stay at previous position
				#ifdef WIN32
					PlaySound("Door-1.au",NULL,SND_ASYNC);
				#else
					system("/usr/sbin/sfplay Door-1.au &");
				#endif
				break;
			case 'P':
				if ((currentX == 0) || (currentY == 0)) // if there is a pedestal on the periphery
					camera.oldXoldY();		//  stay at previous position
				break;
			case ' ':				// in case an orb or a door is on the periphery
				camera.oldXoldY();		// the next position will be a space 
				break;				// so just move back to the previous position
			case 'Z':
				collideWithZorro(currentX, currentY);
				break;
			default:
				break;
		}
		flags.moveForward = FALSE;			// reset the flag
	}
	
	if (flags.turnLeft)					// if j key was pressed
	{	
		flags.turnRight = FALSE;			// reset the turnRight flag
		camera.turnLeft();				// turn Left
		flags.turnLeft = FALSE;				// reset the turnLeft flag
	}	

	if (flags.turnRight)
	{
		flags.turnLeft = FALSE;				// reset the turnLeft flag
		camera.turnRight();				// turn Right
		flags.turnRight = FALSE;			// reset the turnRight flag
	}

	glutTimerFunc(5, update, 1);
    } 


/****************************************************************/
/*  Draw the floor. If textures are enabled then draw the texture*/
/****************************************************************/

void drawFloorWithTexture(int centreX, int centreY)
{
		if (flags.enableTexture)
		{
			glEnable(GL_TEXTURE_2D);
			glBindTexture (GL_TEXTURE_2D, TEXTURE_2);
			drawFloor(centreX, centreY, TRUE);	// draw a floor space
			glDisable(GL_TEXTURE_2D);
		}
		else
			drawFloor(centreX, centreY, FALSE);	// draw a floor space			


}

/****************************************************************/
/*  Depending on what the Maze array character is call the      */
/*  appropriate draw function					*/
/*  Pass the location of the object in the array to the function*/
/****************************************************************/
void chooseWhatToDraw(char c, int centreX, int centreY)
{

	switch(c)
	{
	case ' ':				// a space so don't draw anything
		break;
	case '.':
		drawFloorWithTexture(centreX, centreY);
		break;
	case 'D':
		drawFloorWithTexture(centreX, centreY);
		if (flags.enableTexture)
		{
			glEnable(GL_TEXTURE_2D);
			glBindTexture (GL_TEXTURE_2D, TEXTURE_3);
			drawDoor(centreX, centreY);	// draw a door
			glDisable(GL_TEXTURE_2D);
		}
		else
			drawDoor(centreX, centreY);	// draw a door
		break;		
	case 'O':
		drawFloorWithTexture(centreX, centreY);
		drawOrb(centreX, centreY);	// draw an Orb 
		drawPedestal(c, centreX, centreY); // draw a pedestal also on the same position
		break;		
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
		drawFloorWithTexture(centreX, centreY);
		drawTeleporter(centreX, centreY, c);// draw a teleporter
		break;		
	case 'X':
		drawFloorWithTexture(centreX, centreY);
		startHere(centreX, centreY);	// draw the starting position
		break;		
	case 'P':
		drawFloorWithTexture(centreX, centreY);
		drawPedestal(c, centreX, centreY);	// draw a pedestal
		break;
	case 'Z':
		drawFloorWithTexture(centreX, centreY);
		drawZorro(centreX,centreY);
		break;
	case 'F':
		drawFloorWithTexture(centreX, centreY);
		drawFlutter(centreX, centreY);
		break;		
	default :
		break;
	}
}	


/*************************************************************
 *
 * display
 * 
 * this function is called each time the screen is to be drawn
 *************************************************************/
void display(void)
{

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	if (flags.enableFog)
		enableFog();
	else
		glDisable(GL_FOG);

    /* clear the screen to the clear colour */
    /****************************************/

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    /* position the camera		*/
    /************************************/

	glPushMatrix();
	if (flags.seeFromTop)
	{	
		camera.lookFromTop();
		camera.drawPlayer();
	}
	else
		camera.lookAt();



    /* position the light source	*/
    /************************************/
	
	glLightfv(GL_LIGHT0, GL_POSITION, light_position);

    /* position the ottos		*/
    /************************************/
	if (flags.firstTime)
		for (int i = 0; i < 3; i++)
		{
			positionOtto(i);
			ottoObj[i].drawOtto();
		}
	else
		for (int i = 0; i < 3; i++)
			ottoObj[i].drawOtto();


    /* draw everything */
    /*******************/

	// draw all the walls before you draw the other objects
	// otherwise the lines of other objects are not rendered correctly

	for (int i = 0; i < MAX_LENGTH; i++)
		for (int j = 0; j < MAX_LENGTH; j++)
			if (theMaze[i][j] == '#')
	{
			if (flags.enableTexture)
			{
				glEnable(GL_TEXTURE_2D);
				glBindTexture (GL_TEXTURE_2D, TEXTURE_1);
				drawWall(i, j, TRUE);	// draw a wall
				glDisable(GL_TEXTURE_2D);
			}
			else
				drawWall(i, j, FALSE);	// draw a wall		
	}

	// now draw the other objects
	for ( i = 0; i < MAX_LENGTH; i++)
		for (int j = 0; j < MAX_LENGTH; j++)
			chooseWhatToDraw(theMaze[i][j], i, j);
	flags.firstTime = FALSE;		// after everything is drawn once reset the flag.

	glPopMatrix();
 

	// display the text here. Disable the z buffer and the lighting first and then enable them 
	// after the display is done
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_LIGHTING);
	orbDisplay.setColor(0.0,1.0,0.0);
	orbDisplay.drawText("Orbs taken : ", 1,9); 
	orbDisplay.drawValue(orbDisplay.getOrbCtr(), 10,9);
	ottoDisplay.setColor(0.0,1.0,0.0);
	ottoDisplay.drawText("Ottos taken :",1,7);
	ottoDisplay.drawValue(ottoDisplay.getOttoCtr(),10,7);

	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);


    /* swap buffers */
    /****************/
  
    glutSwapBuffers();
}



/*************************************************************
 *
 * main
 * 
 *************************************************************/

int main(int argc, char **argv)
    {

    /* deal with any GLUT command Line options */
    /*******************************************/
        
	glutInit(&argc, argv);
    
	srand(time(NULL)); // seed the srand function
    /* create an output window */
    /***************************/

	glutInitWindowSize(500, 500);	
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);
	glutCreateWindow("Uberzoik");
	if (glutGet(GLUT_WINDOW_COLORMAP_SIZE) != 0)
	{
		printf("FAIL: bad RGBA colormap size\n");
		exit(1);
	}

	readMaze();			// read the maze	
	flags.firstTime = TRUE;		// set approriate flags
	flags.moveForward = FALSE;
	flags.turnLeft = FALSE;
	flags.turnRight = FALSE;

    /* set up for accepting keyboard commands */
    /******************************************/

    glutKeyboardFunc(keyboard);

    /* set up the logical graphics space */
    /*************************************/

	
	// define the light parameters
	GLfloat white_light[] = {1.0, 1.0, 1.0, 1.0};
	glLightfv(GL_LIGHT0, GL_DIFFUSE, white_light);    

	// enable z buffer
	glEnable(GL_DEPTH_TEST);

	// enable back face culling
	glCullFace(GL_BACK);
	glEnable(GL_CULL_FACE);

	// enable the lighting
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);	

	// load the textures
	// texture 1 for wall
	// texture 2 for floor
	// texture 3 for door
	glGenTextures(1,&TEXTURE_1);
	loadTextureFile ("wall-leaf2.raw", 128,256,TEXTURE_1);

	glGenTextures(1,&TEXTURE_2);
	loadTextureFile ("wall.raw", 128,128,TEXTURE_2);

	glGenTextures(1,&TEXTURE_3);
	loadTextureFile ("grass1.raw", 128,128,TEXTURE_3);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(60, 1, 0.1, 150);
	
	glClearColor(0.0, 0.0, 0.0, 1.0);
	glShadeModel(GL_SMOOTH);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();


    /* assign the display function */
    /*******************************/

	glutDisplayFunc(display);

    /* assign the idle function */
    /****************************/

	glutIdleFunc(display);

    /* set up a timer to regularly update the scene */
    /************************************************/

	glutTimerFunc(5, update, 1);

    /* set everything going */
    /************************/
    
	glutMainLoop();
 
	return 0;  

}

