

#include "common.hxx"
#include "orb.hxx"

// constructor - reads the orb.txt file
orb::orb()
{ 
	int tmp_noOfVertices;
	float tmp_vertex1;
	float tmp_vertex2;
	float tmp_vertex3;
	float tmp_nx;
	float tmp_ny;
	float tmp_nz;

	FILE* fp1 = fopen(orbfile,"r");

	if (!fp1)
	{
		printf("Couldn't open file orb.txt\n");
		exit(1);
	}
	fscanf(fp1,"%d\n", &noOfPolygons);
	fscanf(fp1,"\n");

	fscanf(fp1,"%f %f %f", &colors[0], &colors[1], &colors[2]);
	fscanf(fp1,"\n");
	
	// i keeps track of the number of polygons
	for (int i = 0; i < noOfPolygons; i++)
	{
		fscanf(fp1, "%d", &tmp_noOfVertices);
		noOfVertices.push_back(tmp_noOfVertices);
		fscanf(fp1,"\n");

		// read in the normals 
		// store them in the normals vector
		fscanf(fp1, "%f" "%f" "%f", &tmp_nx, &tmp_ny, &tmp_nz);
		normals.push_back(tmp_nx);
		normals.push_back(tmp_ny);
		normals.push_back(tmp_nz);
		fscanf(fp1,"\n");

		// k tracks the no of vertices
		for (int k =0 ; k < tmp_noOfVertices; k++)
		{
			fscanf(fp1, "%f %f %f", &tmp_vertex1, &tmp_vertex2, &tmp_vertex3);
			vertices.push_back(tmp_vertex1);
			vertices.push_back(tmp_vertex2);
			vertices.push_back(tmp_vertex3);
			fscanf(fp1,"\n");
		}
	}
	fclose(fp1);
	orbTaken = FALSE;	// reset this flag at the beginning

}

// draw the orb at the specified position
void orb::drawOrb(int x, int y)
{
	positionX = x;
	positionY = y;
	angle += 1.5;

	// apply transformations	
	glPushMatrix();
		glTranslatef((x*5.0)+2.5,(y*5.0)+2.5,3.5);	
		glRotatef(angle, 0, 0, 1.0);

		GLfloat mat_diffuse[] = {colors[0], colors[1], colors[2], 1.0};
		glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);

		int i =0;		// i keeps track of the no of coordinates
		int nCtr = 0;		// nCtr keeps track of the normal coordinates for  a polygon
		for (int j = 0; j < noOfPolygons; j++, nCtr += 3)
		{
			glBegin(GL_POLYGON);
				int g=0;	// g tracks no of corners in the polygon
				for ( ; g < noOfVertices[j]; i+=3, g++)
				{
					glNormal3f(normals[nCtr], normals[nCtr+1], normals[nCtr+2]);
					glVertex3f(vertices[i], vertices[i+1], vertices[i + 2]);
				}
			glEnd();
		}
	glPopMatrix();
	
}

//return positionX
int orb::getPositionX()
{
	return positionX;
}

//return positionY
int orb::getPositionY()
{
	return positionY;
}

// set the color to it's maximum intensity
void orb::brightestColor()
{
	if (orbTaken == FALSE)
	{	
		colors[0] = 1.0;
		colors[1] = 1.0;		
		colors[2] = 0.0;		
	}
}

// set the color to it's minimum intensity
void orb::darkestColor()
{
	colors[0] = 0.0;
	colors[1] = 0.0;		
	colors[2] = 0.0;		
}

// set the door color to an intermediate value specified
void orb::changeColor(float difference)
{
	//first check if the orb was taken 
	// if so then teh orb should remain dark
	if (orbTaken == FALSE)
	{
		colors[0] = difference;
		colors[1] = difference;		
		colors[2] = 0.0;		
	}
}	

// set the orbTaken flag
void orb::setOrbTaken()
{
	orbTaken = TRUE;
}

// return orbTaken 
int orb::getOrbTaken()
{
	return orbTaken;
}

//destructor
orb::~orb()
{}
