     

#include "common.hxx"
#include "space.hxx"

// constructor - reads the space.txt file
space::space()
{ 
	int tmp_noOfVertices;
	float tmp_vertex1;
	float tmp_vertex2;
	float tmp_vertex3;
	float tmp_nx;
	float tmp_ny;
	float tmp_nz;

	FILE* fp1 = fopen(spacefile,"r");

	if (!fp1)
	{
		printf("Couldn't open file space.txt\n");
		exit(1);
	}
	fscanf(fp1,"%d\n", &noOfPolygons);
	fscanf(fp1,"\n");

	fscanf(fp1,"%f %f %f", &colors[0], &colors[1], &colors[2]);
	fscanf(fp1,"\n");

	for (int i = 0; i < noOfPolygons; i++)
	{
		fscanf(fp1, "%d", &tmp_noOfVertices);
		noOfVertices.push_back(tmp_noOfVertices);
		fscanf(fp1,"\n");

		// read in the normal direction for the polygon face.
		// and store them in the normals vector	
		fscanf(fp1, "%f %f %f", &tmp_nx, &tmp_ny, &tmp_nz);
		normals.push_back(tmp_nx);
		normals.push_back(tmp_ny);
		normals.push_back(tmp_nz);
		fscanf(fp1,"\n");

		for (int k =0 ; k < tmp_noOfVertices; k++)
		{
			fscanf(fp1, "%f %f %f", &tmp_vertex1, &tmp_vertex2, &tmp_vertex3);
			vertices.push_back(tmp_vertex1);
			vertices.push_back(tmp_vertex2);
			vertices.push_back(tmp_vertex3);
			fscanf(fp1,"\n");
		}
	}
	fclose(fp1);
}

// draw the floor space at the specified position
void space::drawSpace(int x, int y, int status)
{
	positionX = x;
	positionY = y;

	// apply transformations
	glPushMatrix();
		glTranslatef((x*5.0)+2.5,(y*5.0)+2.5,0.0);	

		GLfloat mat_diffuse[4];
		if (status)
		{
			mat_diffuse[0] = 1.0;
			mat_diffuse[1] = 1.0;
			mat_diffuse[2] = 1.0;
			mat_diffuse[3] = 1.0;
		}
		else
		{	
			mat_diffuse[0] = colors[0];
			mat_diffuse[1] = colors[1];
			mat_diffuse[2] = colors[2];
			mat_diffuse[3] = 1.0;
		}

		glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
		GLfloat textureVector[] = {0.0f, 0.0f, 1.0f, 0.0f, 1.0f, 1.0f, 0.0f, 1.0f};
//		GLfloat textureVector[] = {0.0f, 0.0f, 0.5f, 1.0f, 0.5f, 1.0f, 0.0f, 1.0f};		
		int i =0;		// i keeps track of the no of coordinates
		int nCtr = 0;		// nCtr keeps track of the normal coordinates for  a polygon

		for (int j = 0; j < noOfPolygons; j++, nCtr += 3)
		{
			glBegin(GL_POLYGON);
				int g=0;	// g tracks no of corners in the polygon
				int tCtr = 0;	// Texture coordinate counter 
				for ( ; g < noOfVertices[j]; i+=3, g++, tCtr+=2)
				{
					glNormal3f(normals[nCtr], normals[nCtr+1], normals[nCtr+2]);						        				glTexCoord2f (textureVector[tCtr],textureVector[tCtr+1]);
					glVertex3f(vertices[i], vertices[i+1], vertices[i + 2]);
				}
			glEnd();
		}
	glPopMatrix();
	
}

// return positionX
int space::getPositionX()
{
	return positionX;
}

// return positionY
int space::getPositionY()
{
	return positionY;
}

// set the color to it's maximum intensity
void space::brightestColor()
{
	colors[0] = 1.0;
	colors[1] = 1.0;		
	colors[2] = 1.0;		
}

// set the color to it's minimum intensity
void space::darkestColor()
{
	colors[0] = 0.0;
	colors[1] = 0.0;		
	colors[2] = 0.0;		
}

// set the door color to an intermediate value specified
void space::changeColor(float difference)
{
	colors[0] = difference;
	colors[1] = difference;		
	colors[2] = difference;		
}

// destructor
space::~space()
{}
