
	
#include "common.hxx"		// a common header file
#include "viewer.hxx"		// the viewer header file

// Initialize the camera 
void viewer::initialize(int x, int y)
{
	positionX = x;
	positionY = y;

	eyesX = (x * 5.0) + 2.5;		
	eyesY = ((y * 5.0) + 2.5);
	eyesZ = 5.0;
	angle = 0.0;

	currentPositionX = eyesX/5.0;
	currentPositionY = eyesY/5.0;
	
	seeX = ((x - 1) * 5.0) + 2.5;
	seeY = eyesY;
	seeZ = eyesZ;
}

// return eyesX
float viewer::getEyesX()
{
	return eyesX;
}

// return eyesY
float viewer::getEyesY()
{
	return eyesY;
}

// return eyesZ
float viewer::getEyesZ()
{
	return eyesZ;
}

// return seeX
float viewer::getSeeX()
{
	return seeX;
}

// return seeY
float viewer::getSeeY()
{
	return seeY;
}

// return seeZ
float viewer::getSeeZ()
{
	return seeZ;
}


// turn the camera to the left
void viewer::turnLeft()
{
	angle+=2.5;					// change angle	
	seeX = eyesX - (5.0 * cosf(angle * PI/180.0));	// change seeX
	seeY = eyesY - (5.0 * sinf(angle * PI/180.0));  // change seeY
}

// turn the camera to the right
void viewer::turnRight()
{
	angle-=2.5;					// change angle
	seeX = eyesX - (5.0 * cosf(angle * PI/180.0));	// change seeX
	seeY = eyesY - (5.0 * sinf(angle * PI/180.0));	// change seeY
}

// move the camera forward
void viewer::moveForward()
{
	currentPositionX = eyesX/5.0;	// the current positions in the maze array
	currentPositionY = eyesY/5.0;

	currentEyesX = eyesX;		// current position sin the 3D maze
	currentEyesY = eyesY;

	eyesX -= cosf(angle * (PI/180));	// calculate the eyes position
	eyesY -= sinf(angle * (PI/180));
	seeX = eyesX - cosf(angle * (PI/180));	// the see positions shouold also change
	seeY = eyesY - sinf(angle * (PI/180));
			
	nextPositionX = eyesX/5.0;	// next position in the maze array
	nextPositionY = eyesY/5.0;

	nextEyesX = eyesX;		// next position in the 3D maze
	nextEyesY = eyesY;

}

// move to the old X and new Y position
void viewer::oldXnewY()
{
	eyesX = currentEyesX;
	eyesY = nextEyesY;
	seeX = eyesX - cosf(angle * (PI/180));
	seeY = eyesY - sinf(angle * (PI/180));
}

// move to the next X and old Y position
void viewer::newXoldY()
{
	eyesX = nextEyesX;
	eyesY = currentEyesY;
	seeX = eyesX - cosf(angle * (PI/180));
	seeY = eyesY - sinf(angle * (PI/180));
}

// move back to the old X and old Y position
void viewer::oldXoldY()
{
	eyesX = currentEyesX;
	eyesY = currentEyesY;
	seeX = eyesX - cosf(angle * (PI/180));
	seeY = eyesY - sinf(angle * (PI/180));
}

// return currentPositionX
int viewer::getCurrentPositionX()
{
	return currentPositionX;
}

// return currentPositionY
int viewer::getCurrentPositionY()
{
	return currentPositionY;
}

// return nextPositionX
int viewer::getNextPositionX()
{
	return nextPositionX;
}

// return nextPositionY
int viewer::getNextPositionY()
{
	return nextPositionY;
}

// return currentEyesX
float viewer::getCurrentEyesX()
{
	return currentEyesX;
}

// return currentEyesX
float viewer::getCurrentEyesY()
{
	return currentEyesY;
}

// return nextEyesX
float viewer::getNextEyesX()
{
	return nextEyesX;
}

// return nextEyesY
float viewer::getNextEyesY()
{
	return nextEyesY;
}

// set drawZorro
void viewer::setDrawZorro()
{
	drawZorro = TRUE;
}

// change the position of the camera to the values specified
// this is used when you are teleported 
void viewer::changeEyes(int newEyesX, int newEyesY)
{
	eyesX = (newEyesX * 5.0) + 2.5;
	eyesY = ((newEyesY * 5.0) + 2.5);
	seeX = eyesX - cosf(angle * (PI/180));
	seeY = eyesY - sinf(angle * (PI/180));
}

// decide where the camera is positioned and where it is looking at 
void viewer::lookAt()
{
	gluLookAt(eyesX, eyesY, eyesZ, seeX, seeY, seeZ, 0.0, 0.0, 1.0);
}

// when the player switches to the map mode, change the parameters to 
// gluLookAt() 
void viewer::lookFromTop()
{
	gluLookAt(eyesX, eyesY, eyesZ+40, eyesX, eyesY, eyesZ, -1.0, 0.0, 0.0);
}


// this routine draws a rotated cone to indicate  the 
// player in the map mode
void viewer::drawPlayer()
{
	GLfloat mat_diffuse1[] = {1.0, 0.0, 0.0, 1.0};

	glPushMatrix();
		glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse1);
		glTranslatef(eyesX+0.5,eyesY,eyesZ-1);
		glRotatef(angle, 0,0, 1);
		glPushMatrix();
			glRotatef(90.0, 0.0, -1.0, 0.0);
			glutSolidCone(0.5, 1.0, 20, 20);
		glPopMatrix();
	glPopMatrix();
}
